@@ -5,7 +5,12 @@ import java.sql.Connection;
 import java.sql.PreparedStatement;
 import java.sql.SQLException;
 import java.sql.SQLFeatureNotSupportedException;
+import java.sql.Timestamp;
+import java.text.ParseException;
+import java.text.SimpleDateFormat;
 import java.util.Collections;
+import java.util.Date;
+import java.util.TimeZone;
 
 public class BatchedInsert {
 	public static int		INTEGER						= 0;
@@ -16,23 +21,31 @@ public class BatchedInsert {
 	public static int		BIGINT						= 5;
 	
 	public static final int	BIG_DATA_BATCH_INSERT_LIMIT	= 1000;
+
+	private static final String DATE_FORMAT = "yyyy-MM-dd HH:mm:ss";
 	
 	private Object[]		columns;
 	private int[]			columnTypes;
 	private Connection		connection;
+	private String			dbms;
 	private int				columnCount;
 	private int				rowCount;
 	private String			sql;
+	private boolean         supportsAutoCommit;
 	
-	public BatchedInsert(Connection connection, String sql, int columnCount) throws SQLException {
+	public BatchedInsert(Connection connection, String dbms, String sql, int columnCount, boolean supportsAutoCommit) throws SQLException {
 		this.connection = connection;
+		this.dbms = dbms;
 		this.sql = sql;
 		this.columnCount = columnCount;
+		this.supportsAutoCommit = supportsAutoCommit;
 		columns = new Object[columnCount];
 		columnTypes = new int[columnCount];
 	}
 	
 	private void trySettingAutoCommit(boolean value) throws SQLException  {
+		if (!supportsAutoCommit)
+			return;
 		try {
 			connection.setAutoCommit(value);
 		} catch (SQLFeatureNotSupportedException exception) {
@@ -60,7 +73,7 @@ public class BatchedInsert {
 		}
 	}
 	
-	private void setValue(PreparedStatement statement, int statementIndex, int rowIndex, int columnIndex) throws SQLException {
+	private void setValue(PreparedStatement statement, int statementIndex, int rowIndex, int columnIndex) throws SQLException, ParseException {
 		if (columnTypes[columnIndex] == INTEGER) {
 			int value = ((int[]) columns[columnIndex])[rowIndex];
 			if (value == Integer.MIN_VALUE)
@@ -83,8 +96,14 @@ public class BatchedInsert {
 			String value = ((String[]) columns[columnIndex])[rowIndex];
 			if (value == null)
 				statement.setObject(statementIndex, null);
-			else
-				statement.setTimestamp(statementIndex, java.sql.Timestamp.valueOf(value));
+			else {
+				// snowflake driver uses time zone information from client so we need to
+				// use UTC timezone during parsing the value
+				if ("snowflake".equalsIgnoreCase(dbms))
+					setTimestampForSnowflake(statement, statementIndex, value);
+				else
+					statement.setTimestamp(statementIndex, java.sql.Timestamp.valueOf(value));
+			}
 		} else if (columnTypes[columnIndex] == BIGINT) {
 			long value = ((long[]) columns[columnIndex])[rowIndex];
 			if (value == Long.MIN_VALUE)
@@ -99,8 +118,8 @@ public class BatchedInsert {
 				statement.setString(statementIndex, value);
 		}
 	}
-	
-	public boolean executeBatch() throws SQLException {
+
+	public boolean executeBatch() throws SQLException, ParseException {
 		checkColumns();
 		try {
 			trySettingAutoCommit(false);
@@ -129,7 +148,7 @@ public class BatchedInsert {
 	 * insert with multiple values.
 	 * @throws SQLException 
 	 */
-	public boolean executeBigQueryBatch() throws SQLException {
+	public boolean executeBigQueryBatch() throws SQLException, ParseException {
 		checkColumns();
 		try {
 			trySettingAutoCommit(false);
@@ -241,4 +260,11 @@ public class BatchedInsert {
 	public void setBigint(int columnIndex, double column) {
 		setBigint(columnIndex, new double[] { column });
 	}
+
+	private static void setTimestampForSnowflake(PreparedStatement statement, int statementIndex, String value) throws ParseException, SQLException {
+		SimpleDateFormat sdf = new SimpleDateFormat(DATE_FORMAT);
+		sdf.setTimeZone(TimeZone.getTimeZone("UTC"));
+		Date date = sdf.parse(value);
+		statement.setTimestamp(statementIndex, new Timestamp(date.getTime()));
+	}
 }
